function M=fliptree(nX,nY,m,Tsplit,alpha,Tgrowth,N0X,N0Y,Nadd,Tmass,Pmass)
%function M=fliptree(nX,nY,m,Tsplit,alpha,Tgrowth,N0X,N0Y,Nadd,Tmass,Pmass)
%Allows a one-way admix event to occur at time Tmass, when a prop. Pmass of PopY migrates to X
%
%Corrects for cases where Npairs is large compared to Nt, by allowing >1 coalescent between gens
%Simulates coalescent tree as M where:
%First n rows = terminal nodes
%Next (n-1) rows = internal nodes.
%First col = time (going backwards) in units of (N=total pop size), since previous coalescent event
%2nd col =  index of father
%3rd col = index of right descendent
%4th col = index of left descendent
%nX, nY: No. sampled from each pop
%Tsplit: time of split (in gens)
%Tgrowth: time exponential growth starts (in gens)
%Tmass: time of mass migration event (in gens)
%N0X, N0Y:  initial pop size.  Nt = N0*exp(alpha*(Tgrowth-t)) if t<Tgrowth, else Nt=N0
%Nadd: allows an additional "Nadd" to be added to total pop. at time of joining.
%alpha: growth rate/gen -same in both pops
%m: symmetric migration rate: prop. of X that goes to Y and of Y that goes to X. = P(migration)
%Pmass: Fraction of PopY that migrates to PopX at time Tmass

totn=nX+nY;
M=zeros(2*totn-1,4);
X=(1:nX);    		%stores list of indices for currently "active" lines in X
Y=(nX+1:nX+nY);		%stores list of indices for currently "active" lines in Y
nXpairs=nX*(nX-1)/2;
nYpairs=nY*(nY-1)/2;
NtX=N0X*exp(alpha*Tgrowth);
NtY=N0Y*exp(alpha*Tgrowth);

T=0; i=totn; 			%i stores current index
mixes=0;
while (nX+nY)>1
  T=T+1;
  if T==Tmass
    %note that going backwards in time, linneages in X "migrate" to Y
    %We assume automatic readjustment of both pop sizes to orig level after flux event
    Xmig = (rand(1,nX)<Pmass);
    Y = [Y X(Xmig)];  nY=length(Y); nYpairs=nY*(nY-1)/2;
    X = X(~Xmig);  nX=length(X); nXpairs=nX*(nX-1)/2;
  end
  if T<=Tgrowth
    NtX=N0X*exp(alpha*(Tgrowth-T));
    NtY=N0Y*exp(alpha*(Tgrowth-T));
  end
  smallt = -log(rand)*NtX;               %gives time to next coal event in units of gen/nXpairs
  while smallt<nXpairs                   %events must happen within this single gen
    i=i+1;
    %choose descendents
    d1 = X(ceil(rand*nX));  X = X(X~=d1);
    nX=nX-1;
    d2 = X(ceil(rand*nX));  X(X==d2) = i;
    %Update M
    M(i,1) = T-1+smallt/nXpairs;
    M(i,3)=d1;   M(i,4)=d2;
    M(d1,2)=i;  M(d2,2)=i;
    nXpairs = nXpairs - nX;
    smallt = smallt -log(rand)*NtX;
  end
  %now to check for Y and for migration events, given T is before split
  if T<Tsplit
    smallt = -log(rand)*NtY;          %gives time to next coal event in units of gen/nYpairs
    while smallt<nYpairs              %events must happen within this single gen
      i=i+1;
      %choose descendents
      d1 = Y(ceil(rand*nY));  Y = Y(Y~=d1);
      nY=nY-1;
      d2 = Y(ceil(rand*nY));  Y(Y==d2) = i;
      %Update M
      M(i,1) = T-1+smallt/nYpairs;
      M(i,3)=d1;   M(i,4)=d2;
      M(d1,2)=i;  M(d2,2)=i;
      nYpairs = nYpairs - nY;
      smallt = smallt -log(rand)*NtY;
    end
    %do migration events
    Xmig = (rand(1,nX)<m);  Xtrans = X(Xmig);
    Ymig = (rand(1,nY)<m);  Ytrans = Y(Ymig);
    X = [X(~Xmig) Ytrans];  nX=length(X); nXpairs=nX*(nX-1)/2;
    Y = [Y(~Ymig) Xtrans];  nY=length(Y); nYpairs=nY*(nY-1)/2;
  elseif T==Tsplit			%time to merge all lineages into X
    X = [X Y]; nX=nX+nY; nXpairs=nX*(nX-1)/2;
    Y = []; nY=0; nYpairs=0;
    N0X=N0X+N0Y+Nadd; NtX=N0X*exp((T<Tgrowth)*alpha*(Tgrowth-T));   %pops have merged.
  end  %of if T<Tsplit
end

